c       .................................................................
c       .  Scattering program for a bisphere.  
c It is based on the derivation given in
c Videen, Ngo and Hart, Opt. Comm 125, p. 275 (1996)
c except the refractive indices of the spheres are not restricted bo
c being perfectly conducting. This versions produces Mueller matrices,
c efficiencies -- comments are placed in appropriate locations
c to maximize efficiency. The code was written by Dat Ngo, so it is
c fairly well commented. If there are any questions, I can be reached
c at
c videen@atm.dal.ca
c
c       .  Input is from "bulb.in", and declarations in "case1.def".    .
c       .................................................................
	implicit double precision (a-h,o-z)
	INCLUDE 'case1.def'

	open(unit=4,file='bulb.in',status='old')
	rewind(4)
	open(unit=8,file='bulb.out')
c  ..........................................................
c  .  read in the input values from "bulb.in"               .
c  ..........................................................
	read(4,*) wavel
	read(4,*) rad1
	read(4,*) rad2
	read(4,*) distance
	read(4,*) mr_1, mi_1
	read(4,*) mr_2, mi_2
	read(4,*) inc_ang
	read(4,*) num
	read(4,*) phi

	pi=1.0d0
	ccc=dcmplx(0.0,pi)
	pi=4.0*datan(pi)

	k0=2.*pi/wavel
	m_1=dcmplx(mr_1,mi_1)
	m_2=dcmplx(mr_2,mi_2)
	k1=k0*m_1
	k2=k0*m_2
	print*,'k1,k2=',k1,k2
	x01=k0*rad1
	x1=k1*rad1
	x02=k0*rad2
	x2=k2*rad2
	xd=k0*distance

c	print*,'x01=',x01
c	print*,'x02=',x02
c	print*,'xd=',xd
      nbg=cdabs(xd) + 4.*cdabs(xd)**.333 + 2.0
c      print*,'the value of nbg=',nbg
c	read(5,*)nbg
c .......................................................................
c .    Calculate Riccati-Bessel Functions and Derivatives needed        .
c .......................................................................
      call c_bessel(nbg,isize,besj1,x01)
      call riccati(nbg,besj1,psi01,dpsi01,x01)      
      call c_bessel(nbg,isize,besj2,x02)        
      call riccati(nbg,besj2,psi02,dpsi02,x02)  
      call c_newman(nbg,isize,hankel1,x01)
      call c_newman(nbg,isize,hankel2,x02)
      do i=-1,nbg
	hankel1(i)=besj1(i) + ccc*hankel1(i)
	hankel2(i)=besj2(i) + ccc*hankel2(i)
      enddo                                  
      call riccati(nbg,hankel1,zeta01,dzeta01,x01)
      call riccati(nbg,hankel2,zeta02,dzeta02,x02)        

      call c_bessel(nbg,isize,besj1,x1)
      call c_newman(nbg,isize,hankel1,x1)
      call c_bessel(nbg,isize,besj2,x2)
      call c_newman(nbg,isize,hankel2,x2)
      do 12 i=-1,nbg
	hankel1(i)=besj1(i) + ccc*hankel1(i)
	hankel2(i)=besj2(i) + ccc*hankel2(i)
 12   continue                                  
	call riccati(nbg,besj1,psi1,dpsi1,x1)
	call riccati(nbg,hankel1,zeta1,dzeta1,x1)
	call riccati(nbg,besj2,psi2,dpsi2,x2)
	call riccati(nbg,hankel2,zeta2,dzeta2,x2)

c ....................................................
c .  Now we must solve for the constants Gamma's     .
c ....................................................
	 do 25 n=0,nbg
	    fact1=k1*psi01(n)*dpsi1(n) - k0*psi1(n)*dpsi01(n)
	    fact2=k1*zeta01(n)*dpsi1(n)-k0*psi1(n)*dzeta01(n)
	    Gam1(n)=fact1/fact2
	    fact1=k0*psi01(n)*dpsi1(n) - k1*psi1(n)*dpsi01(n)
	    fact2=k0*zeta01(n)*dpsi1(n)-k1*psi1(n)*dzeta01(n)
	    Gam2(n)=fact1/fact2

	    fact1=k2*psi02(n)*dpsi2(n) - k0*psi2(n)*dpsi02(n)
	    fact2=k2*zeta02(n)*dpsi2(n)-k0*psi2(n)*dzeta02(n)
	    Gam3(n)=fact2/fact1
	    fact1=k0*psi02(n)*dpsi2(n) - k2*psi2(n)*dpsi02(n)
	    fact2=k0*zeta02(n)*dpsi2(n)-k2*psi2(n)*dzeta02(n)
	    Gam4(n)=fact2/fact1
 25      continue

c .......................................................................
c .    Calculate translation coefficients when the separation of the    .
c .    center to center radii is d.  We will first find the bessel      .
c .    function which describes the separation, k0*d                    .
c .......................................................................
	call c_bessel(4*nbg,isiz4,besj_d,xd)
	call c_newman(4*nbg,isiz4,hankeld,xd)
	do i=-1,4*nbg
		besj_d(i)=besj_d(i) + ccc*hankeld(i)
	enddo
c ...............................................
c .    using 4.23 and 4.24 from the paper first .
c .    keeping in mind that cuplo = c0,0(np) and.
c .    cuplo1 = c-1,0(np) for starters          .
c ...............................................
c       do 998 loop_a=0,180,18
c       inc_ang=dfloat(loop_a)

      do 110 np=0,nbg*4
	fnp=dfloat(np)
	cuplo(np)=besj_d(np)*dsqrt(2.*fnp+1.)
	cuplo1(np)=-cuplo(np)
 110  continue
      do 115 np=0,nbg*2
	cuplom(0,np)=cuplo(np)          
 115  continue

      do 140 n=1,nbg*2
	fn=dfloat(n)
	do 120 np=0,nbg*4-n
	  fnp=dfloat(np)
	  c=cdsqrt(dcmplx((2.*fnp+1.)/(2.*fn-3.)))*(fn-1.)
	  c=cuplo1(np)*c
	  cuplo1(np)=cuplo(np)
	  cuplo(np)=c
 120    continue

	do 130 np=0,nbg*4-n
	  fnp=dfloat(np)
	  c=-(fnp+1.)*dsqrt((2.*fn-1.)/(2.*fnp+3.))*cuplo1(np+1)
       c=c+fnp*cdsqrt(dcmplx((2.*fn-1.)/(2.*fnp-1.)))*cuplo1(np-1)
	  c=c+cuplo(np)
	  cuplo(np)=c*dsqrt((2.*fn+1.)/(2.*fnp+1.))/fn

	  if (np.le.nbg*2) then
	    cuplom(n,np)=cuplo(np) 
	  endif
 130    continue
 140  continue
c ...................................................
c . Now that we have the elements in the matrix     .
c . cuplom(n,np), we can start the iterations in    .
c . m to get the rest of the translation coeff      .
c . Then we'll have the complete set of C(n,m)np    .
c ...................................................
c	do 299 m=0,nbg
	do 299 m=1,1
	    fact3=cdexp(ccc*xd*dcosd(inc_ang))
c       ......................................................
c       . But first we need to calculate, for each m, the    .
c       . Legendre Polynomials for incident angle theta      .
c       ......................................................
	    ii=nbg+1
	    x=dcos(pi*inc_ang/180.)
	    mmm=m+1
	    call nplgndr(mmm,ii,isize1,legpol1,x)
	    mmm=m
	    call nplgndr(mmm,ii,isize1,legpol2,x)
	    if (m.gt.0) then
	       mmm=m-1
	       call nplgndr(mmm,ii,isize1,legpol,x)
	    else
	       do 165, jjj=-2,isize1
		  legpol(jjj)=-legpol1(jjj)
 165           continue
	    endif

	    do 180 i=m,nbg
	       fi=dfloat(i)
	       tau=dsqrt((fi+m+1.)*(fi-m))*legpol1(i)
	       pie=tau
	       tau=(tau-dsqrt((fi+m)*(fi-m+1.))*legpol(i))/2.0
	       if (dabs(x).gt.0.5) then
c       .................................................
c       . The angle theta is between 0 and 60 degrees   .
c       .................................................
		 pie=pie+dsqrt((fi+m)*(fi-m+1.))*legpol(i)
		 pie=pie*.5/x
	       else
c       .................................................
c       . theta is between 60 and 90; use this so the   .
c       . Polynomial doesn't blow up at 90 or 0 degrees .
c       .................................................
		 pie=m*legpol2(i)/dsin(inc_ang*pi/180.)
	       endif
	       if (i.gt.0) then
		c = (ccc**i)/(fi*(fi+1.))
		aa1(i)=c*tau     
		aa2(i)=-c*pie    
		ee1(i)=aa1(i)*fact3
		ee2(i)=aa2(i)*fact3
	       endif
c       if(m.eq.0) write(8,*)i,aa1(i),aa2(i)                      
c       if(m.le.1) print*,m,i,real(pie),real(tau)
	aaa1(m,i)=aa1(i)
	aaa2(m,i)=aa2(i)
 180        continue
c ...............................................................
c .     That ends the Calculation for the Legendre Polynomials  .
c .     for each value of m.  The incident field coefficients   .
c .     for the TE case are stored in aa1(i) and aa2(i); the TM .
c .     case will be done later by using eqns 37 & 38           .
c ...............................................................

c ...............................................................
c . Now to find the rest of the matrix in C(n,m)np.  For each   .
c . value of m that is greater than 0, say 1, we can find the   .
c . matrix C(n,1)np and store that into cuplom(n,np); then      .
c . the next iteration of m, i.e. m=2, we can find C(n,2)np     .
c . by already having C(n,1)np stored in the matrix cuplom(n,np).
c . Use equation 4.26                                           .
c ...............................................................
      if (m.gt.0) then
	do 155 n=m-1,2*nbg-m+1
		do 154 np=m-1,2*nbg-m+1
			cuplom1(n,np)=cuplom(n,np)
 154            continue
 155    continue

	do 160 n=m,2*nbg-m
	  fn=dfloat(n)
	  do 159 np=m,2*nbg-m
	   fnp=dfloat(np)
       c=dsqrt((fnp-m+1.)*(fnp+m)*(2.*fnp+1.))*cuplom1(n,np)
	   cuplom(n,np)=c
	   c=xd*dsqrt((fnp-m+2.)*(fnp-m+1.)/(2.*fnp+3.))
	   cuplom(n,np)=cuplom(n,np)-c*cuplom1(n,np+1)
	   c=xd*dsqrt((fnp+m)*(fnp+m-1.)/(2.*fnp-1.))
	   cuplom(n,np)=cuplom(n,np)-c*cuplom1(n,np-1)
	   c=dsqrt((fn-m+1.)*(fn+m)*(2.*fnp+1.))
	   cuplom(n,np)=cuplom(n,np)/c
c	if(m.eq.1)write(8,*)n,np,cuplom(n,np)
 159      continue
 160    continue
      endif
c ............................................................
c . Now that we have the matrix which contains the C(n,m)np  .
c . we can go ahead and find the translation coefficients    .
c . A(n,m)np and B(n,m)np by using eqns 4.21 & 4.22. Note,   .
c . we do need to store TransAm and TransBm into a matrix    .
c . like we did for the C(n,m)np for later manipulation.     .
c . The arrays InvAm and InvBm are the translation matrix    .
c . for a translation in the -d direction.  Since the fnct   .
c . is a Bessel function, there is a simple rule to get the  .
c . Inverse of the translation (i.e. InvAm and InvBm)        .
c ............................................................
      do 190 n=m,nbg
	 pie=0.0
	 tau=0.0
	 do 195 np=m,nbg
	    fnp=dfloat(np)
	    tau=(fnp-m+1.0)*(fnp+m+1.0)
	    tau=tau/((2.*fnp+1.)*(2.*fnp+3.))
	    tau=dsqrt(tau)
	    TransA=-xd/(fnp+1.0)
	    TransA=TransA*tau*cuplom(n,np+1)
	    TransB=0.0
	    if (np.gt.0) then
	     pie=(fnp-m)*(fnp+m)
	     pie=pie/((2.*fnp-1.)*(2.*fnp+1.))
	     pie=dsqrt(pie)
	     c=-xd*pie*cuplom(n,np-1)/fnp
	     TransA=TransA+c
	     TransB=-ccc*xd*m*cuplom(n,np)/(fnp*(fnp+1.))
	    endif
	    TransA=cuplom(n,np)+TransA
	    TransAm(n,np)=TransA
	    TransBm(n,np)=TransB
	    if(mod(n+np,2).eq.0) then
		InvAm(n,np)=TransAm(n,np)
		InvBm(n,np)=-TransBm(n,np)
	    else
		InvAm(n,np)=-TransAm(n,np)
		InvBm(n,np)=TransBm(n,np)
	    endif
c       if(n.eq.np) write(8,*)n,np,TransAm(n,np),TransBm(n,np)
c       print*,InvAm(n,np),InvBm(n,np)
 195     continue
 190  continue

c ............................
c . Now to FILL the MATRIX   .
c ............................
      numel=nbg-m+1
      numtot=numel+numel
      do 220 n=m,nbg
	i=n-m+1
	do 215 npp=m,nbg
	   s1=0.0
	   s2=0.0
	   s3=0.0
	   s4=0.0
	   do 214 np=m,nbg
		fact1=TransAm(npp,np)*InvAm(np,n)*Gam1(np)
		fact2=TransBm(npp,np)*InvBm(np,n)*Gam2(np)
		s1=s1+fact1+fact2

		fact3=TransAm(npp,np)*InvBm(np,n)*Gam2(np)
		fact4=TransBm(npp,np)*InvAm(np,n)*Gam1(np)
		s2=s2+fact3+fact4

		fact1=TransAm(npp,np)*InvBm(np,n)*Gam1(np)
		fact2=TransBm(npp,np)*InvAm(np,n)*Gam2(np)
		s3=s3+fact1+fact2

		fact3=TransBm(npp,np)*InvBm(np,n)*Gam1(np)
		fact4=TransAm(npp,np)*InvAm(np,n)*Gam2(np)
		s4=s4+fact3+fact4
 214       continue
	      if(n.eq.npp) then
		s1 = s1 - Gam3(n)
		s4 = s4 - Gam4(n)
	      endif
	      ii=npp-m+1
	      matrixlu(i,ii)=s1
	      matrix(i,ii)=s1
c       print*,i,ii,matrix(i,ii)         

	      matrixlu(i,ii+numel)=s2
	      matrix(i,ii+numel)=s2

	      matrixlu(i+numel,ii)=s3
	      matrix(i+numel,ii)=s3

	      matrixlu(i+numel,ii+numel)=s4
	      matrix(i+numel,ii+numel)=s4
 215        continue
 220  continue
c................................................................
c. THE MATRIX IS FILLED!!  That was the heart of the problem    .
c. Now to perform the LUDCMP of that giant ugly Matrix          .
c................................................................
	ii=isiz2
	call ludcmp(matrixlu,numtot,ii,indx,dd)
c       goto 777        
c	print*,'doing TE mode'
c .............
c For TE case .
c ........................................
c . Now to load up the LHS of the matrix .
c ........................................
	do 226 n=m,nbg
	   sum1=0.0
	   sum2=0.0
	   do 225 np=m,nbg
	      fact1=aa1(np)*InvAm(np,n)*Gam1(np)
	      fact2=aa2(np)*InvBm(np,n)*Gam2(np)
	      sum1=sum1 + fact1 + fact2

	      fact3=aa1(np)*InvBm(np,n)*Gam1(np)
	      fact4=aa2(np)*InvAm(np,n)*Gam2(np)
	      sum2=sum2 + fact3 + fact4
 225       continue
	   i=n-m+1
	   b(i)=ee1(n) - sum1
	   b(i+numel)=ee2(n) -sum2
	   bd(i)=b(i)
	   bd(i+numel)=b(i+numel)
c       print*,'LHS=',i,b(i)
 226    continue
c ....................
c . SOLVE the MATRIX .
c ....................
	ii=isiz2
	call lubksb(matrixlu,numtot,ii,indx,b)
	call mprove(matrix,matrixlu,numtot,ii,indx,bd,b)
	do 227 n=m,nbg
	      i=n-m+1
	      ff3TE(m,n)=b(i)
	      ff4TE(m,n)=b(i+numel)
 	      cc3te(m,n)=ff3TE(m,n)*cdexp(-ccc*xd*dcosd(inc_ang))
              cc4te(m,n)=ff4TE(m,n)*cdexp(-ccc*xd*dcosd(inc_ang))
c       print*,'SCAT ff4te=',i+numel,b(i+numel)
 227    continue

c +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c .  Now we must find the Scattering coefficients bb3TE and bb4TE,      .
c .  and from these coeff we can determine the scattering matrices.     .
c .......................................................................
	do 250 n=m,nbg
	   sum1=0.0
	   sum2=0.0
	   do 260 np=m,nbg
	      fact1=TransAm(np,n)*ff3TE(m,np)
	      fact2=TransBm(np,n)*ff4TE(m,np)
	      sum1=fact1+fact2+sum1
	      fact3=TransBm(np,n)*ff3TE(m,np)
	      fact4=TransAm(np,n)*ff4TE(m,np)
	      sum2=sum2+fact3+fact4
 260       continue
	   Cnm1(m,n)=sum1
	   Cnm2(m,n)=sum2
	   bb3TE(m,n)=-(aa1(n)+sum1)*Gam1(n)
	   bb4TE(m,n)=-(aa2(n)+sum2)*Gam2(n)
c       print*,'Scat bb4te=',n,bb4TE(m,n)
 250    continue

 	do n=m,nbg
           sum1=0.0
           sum2=0.0
           do np=m,nbg
              fact1=InvAm(np,n)*bb3TE(m,np)
              fact2=InvBm(np,n)*bb4TE(m,np)
              sum1=fact1+fact2+sum1
              fact3=InvBm(np,n)*bb3TE(m,np)
              fact4=InvAm(np,n)*bb4TE(m,np)
              sum2=sum2+fact3+fact4
           enddo
           Gnm1(m,n)=sum1
           Gnm2(m,n)=sum2
        enddo

c       MAKE A CALL TO FIND cc3TE and cc4TE here!!! 
c       goto 777        
c 777   continue
c	print*,'doing TM mode'
c.................  
c FOR TM CASE!!! .         
c.................................................................
c Now to load up the LHS, remember that aa1 and aa2 are switched .
c.................................................................
	 do 229 n=m,nbg
	    sum1=0.0
	    sum2=0.0
	    do 228 np=m,nbg
		fact1=ccc*aa2(np)*InvAm(np,n)*Gam1(np)
		fact2=ccc*aa1(np)*InvBm(np,n)*Gam2(np)
		sum1=sum1 + fact1 + fact2
		fact3=ccc*aa2(np)*InvBm(np,n)*Gam1(np)
		fact4=ccc*aa1(np)*InvAm(np,n)*Gam2(np)
		sum2=sum2 + fact3 + fact4
 228        continue
	    i=n-m+1
	    b(i)=ccc*ee2(n) - sum1
	    b(i+numel)=ccc*ee1(n)-sum2
	    bd(i)=b(i)
	    bd(i+numel)=b(i+numel)
 229     continue
c .....................
c . SOLVE the MATRIX  .
c .....................
	ii=isiz2
	call lubksb(matrixlu,numtot,ii,indx,b)
	call mprove(matrix,matrixlu,numtot,ii,indx,bd,b)
	do 230 n=m,nbg
	      i=n-m+1
	      ff3TM(m,n)=b(i)
	      ff4TM(m,n)=b(i+numel)
       	      cc3tm(m,n)=ff3TM(m,n)*cdexp(-ccc*xd*dcosd(inc_ang))
              cc4tm(m,n)=ff4TM(m,n)*cdexp(-ccc*xd*dcosd(inc_ang))
c       print*,'Scat ff4tm=',i+numel,b(i+numel)
 230    continue

c +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c .  Now we must find the Scattering coefficients bb3TM and bb4TM,      .
c .  and from these coeff we can determine the scattering matrices.     .
c .......................................................................
	do 251 n=m,nbg
	   sum1=0.0
	   sum2=0.0
	   do 261 np=m,nbg
	      fact1=TransAm(np,n)*ff3TM(m,np)
	      fact2=TransBm(np,n)*ff4TM(m,np)
	      sum1=fact1+fact2+sum1
	      fact3=TransBm(np,n)*ff3TM(m,np)
	      fact4=TransAm(np,n)*ff4TM(m,np)
	      sum2=sum2+fact3+fact4
 261       continue
	   Cnm1(m,n)=sum1
	   Cnm2(m,n)=sum2
	   bb3TM(m,n)=-(ccc*aa2(n)+sum1)*Gam1(n)
	   bb4TM(m,n)=-(ccc*aa1(n)+sum2)*Gam2(n)
c       print*,'Scat bb4TM',n,bb4TM(m,n)
 251    continue

c ....................................
c . Now to find Cext for this system .
c ....................................
        do 510 n=1,nbg
           s1=(bb3TE(m,n) + cc3te(m,n))*dconjg(aa1(n))
           s1=s1 + (bb3TM(m,n) + cc3tm(m,n))*dconjg(ccc*aa2(n))
           s2=(bb4TE(m,n) + cc4te(m,n))*dconjg(aa2(n))
           s2=s2 + (bb4TM(m,n) + cc4tm(m,n))*dconjg(ccc*aa1(n))
           s3=n*(n+1)*(s2+s1)
           if (m.eq.0) then
                s3=-2.0*s3
           else
                s3=-4.0*s3
           endif
           Cext=Cext + dreal(s3)
 510    continue
c ........................................................................
c .  That finishes one loop in m, now we move to the next m and repeat   .
c .  the same procedure all over again.                                  .
c ........................................................................
c 777   continue
 299   continue
	Cext=Cext/cdabs(k0)**2
	print*,'cext=',cext
	qext=Cext/rad1**2
	print*,'qext=',qext
c............................................
c DONE!!  The sum over m is now complete!!! .
c............................................
	
c .....................................................................
c . Here's the meat of the problem.  We will calculate the amplitude  .
c . scattering matrices for the fields that we've found.  The angle   .
c . theta is our observation angle.  Num is the number of angles we   .
c . want in determining the scattering intensity.  The rest of this   .
c . should look almost like the Mie Code found in Bohren and Huffman  .
c . If we want to have the observation angle's resolution of 1 degree .
c . then set num=360, if we want finer resolution, let num be a bigger.
c . number such as 720 (to have resolution of 0.5degrees)             .
c .....................................................................

	phi_o=dfloat(phi*pi/180.0)
	i=0
	b_angle=inc_ang+180.0
	f_angle=inc_ang
	d_ang=360.0/num

	do 399 j=1,num+1
	 angle=d_ang*float(j-1)
	 angledd=angle
	 if((angle.ge.f_angle).and.(angle.le.b_angle)) then
c	 if((angle.eq.b_angle).or.(angle.eq.f_angle)) then
	print*,'angle=',angledd
	    if(angle.gt.180.0) then
		angle=dabs(360.0-angle)
		phi=pi + phi_o
	    else
		phi=phi_o
	    endif
	    i=i+1
	    s1TE(i)=0.
	    s3TE(i)=0.
	    s2TM(i)=0.
	    s4TM(i)=0.

	    theta=angle*pi/180.
	    x=dcos(theta)

	    do 330 m=0,nbg
c       ...........................................................
c       .  The next few lines will calc the Legendre polynomial   .
c       .  at the angle theta                                     .
c       ...........................................................
		  ii=nbg+1
		  mmm=m
		  call nplgndr(mmm,ii,isize1,legpol2,x)
		  mmm=m+1
		  call nplgndr(mmm,ii,isize1,legpol1,x)
		  if (m.eq.0) then
			do 305 jj=-2,isize1
			      legpol(jj)=-legpol1(jj)
 305                    continue
		  else
			mmm=m-1
			call nplgndr(mmm,ii,isize1,legpol,x)
		  endif

		  do 310 n=m,nbg      
c       .........................................................
c       .  Calculate the corresponding pie and tau              .
c       .........................................................
		    tau=sqrt((n+m+1.)*(n-m))*legpol1(n)
		    pie=tau
		    tau=-(tau-sqrt((n+m)*(n-m+1.))*legpol(n))/2.
		    if (dabs(x).gt.0.5) then
		      pie=pie+sqrt((n+m)*(n-m+1.))*legpol(n)
		      pie=pie*.5/x
		    else
		      pie=m*legpol2(n)/dsin(theta)
		    endif
c ..................................................................
c .Done with finding Tau and Pie at each angle THETA; likewise,    .
c .need to find the scattering amplitude S1,S2,S3,and S4 at THETA  .
c ..................................................................
		    ifact=(-ccc)**n
		    if(m.eq.0) then
		      sum1=bb4TE(0,n)+cdexp(-ccc*xd*x)*ff4TE(0,n)
		      sum2=bb3TE(0,n)+cdexp(-ccc*xd*x)*ff3TE(0,n)
		      fact1=sum1*pie + sum2*tau

		      sum1=bb3TM(0,n)+cdexp(-ccc*xd*x)*ff3TM(0,n)
		      sum2=bb4TM(0,n)+cdexp(-ccc*xd*x)*ff4TM(0,n)
		      fact2=sum1*pie + sum2*tau

		      sum1=bb3TE(0,n)+cdexp(-ccc*xd*x)*ff3TE(0,n)
		      sum2=bb4TE(0,n)+cdexp(-ccc*xd*x)*ff4TE(0,n)
		      fact3=sum1*pie + sum2*tau

		      sum1=bb4TM(0,n)+cdexp(-ccc*xd*x)*ff4TM(0,n)
		      sum2=bb3TM(0,n)+cdexp(-ccc*xd*x)*ff3TM(0,n)
		      fact4=sum1*pie + sum2*tau
	      
		      s1=ifact*fact1
		      s2=ifact*fact2
		      s3=ifact*fact3
		      s4=ifact*fact4
		    else
		      sum1=bb4TE(m,n)+cdexp(-ccc*xd*x)*ff4TE(m,n)
		      sum2=bb3TE(m,n)+cdexp(-ccc*xd*x)*ff3TE(m,n)
		      fact1=sum1*pie + sum2*tau
		      fact1=fact1*2.0*dcos(m*phi)

		      sum1=bb3TM(m,n)+cdexp(-ccc*xd*x)*ff3TM(m,n)
		      sum2=bb4TM(m,n)+cdexp(-ccc*xd*x)*ff4TM(m,n)
		      fact2=sum1*pie + sum2*tau
		      fact2=fact2*2.0*dcos(m*phi)

		      sum1=bb3TE(m,n)+cdexp(-ccc*xd*x)*ff3TE(m,n)
		      sum2=bb4TE(m,n)+cdexp(-ccc*xd*x)*ff4TE(m,n)
		      fact3=sum1*pie + sum2*tau
		      fact3=fact3*2.0*ccc*dsin(m*phi)

		      sum1=bb4TM(m,n)+cdexp(-ccc*xd*x)*ff4TM(m,n)
		      sum2=bb3TM(m,n)+cdexp(-ccc*xd*x)*ff3TM(m,n)
		      fact4=sum1*pie + sum2*tau
		      fact4=fact4*2.0*ccc*dsin(m*phi)

		      s1=ifact*fact1
		      s2=ifact*fact2
		      s3=ifact*fact3
		      s4=ifact*fact4
		    endif
		    s1TE(i)=s1TE(i)+s1 
		    s2TM(i)=s2TM(i)+s2
		    s3TE(i)=s3TE(i)+s3
		    s4TM(i)=s4TM(i)+s4
 310              continue
c ..............................................
c . Finished summing over n (loop 310); now to .
c . sum over m (loop 330) to complete the S's  .
c ..............................................
 330        continue
	    s1TE(i)=s1TE(i)
	    s2TM(i)=-ccc*s2TM(i)
	    s3TE(i)=-ccc*s3TE(i)
	    s4TM(i)=s4TM(i)
c ..............................................
c . Done summing over m.  Now we have one       .
c . complete set of scat amp S(i)               .
c ..............................................
c +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c .  Lastly, we can now use the scattering amplitude matrix     .
c .  and solve for the Mueller Matrix elements.  Once we have   .
c .  the MM, we have all the informations we need.              .
c +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c       write(9,*)'angle    S11       S12             S33                S34'
	do 410 jj=1,4
	    mm(1,jj)=0.0
	    mm(2,jj)=0.0
	    mm(3,jj)=0.0
	    mm(4,jj)=0.0
 410    continue
	    fact1=s1TE(i)*dconjg(s1TE(i))/2.
	    fact2=s2TM(i)*dconjg(s2TM(i))/2.
	    fact3=s3TE(i)*dconjg(s3TE(i))/2.
	    fact4=s4TM(i)*dconjg(s4TM(i))/2.
	    c=s2TM(i)*dconjg(s3TE(i))
	    cc=s1TE(i)*dconjg(s4TM(i))
	    mm(1,1)=dreal(fact1+fact2+fact3+fact4)
	print*,real(angle),real(mm(1,1))
	    mm(1,2)=dreal(fact2-fact1-fact3+fact4)/mm(1,1)
	    mm(1,3)=dreal(c + cc)/mm(1,1)
	    mm(1,4)=dimag(c -cc)/mm(1,1)

	    mm(2,1)=dreal(fact2-fact1+fact3-fact4)
	    mm(2,2)=dreal(fact2+fact1-fact3-fact4)/mm(1,1)
	    mm(2,3)=dreal(c -cc)/mm(1,1)
	    mm(2,4)=dimag(c+cc)/mm(1,1)

	    fact1=s2TM(i)*dconjg(s4TM(i))
	    fact2=s1TE(i)*dconjg(s3TE(i))
	    fact3=s1TE(i)*dconjg(s2TM(i))
	    fact4=s3TE(i)*dconjg(s4TM(i))
	    c=s2TM(i)*dconjg(s1TE(i))
	    cc=s4TM(i)*dconjg(s3TE(i))
	    mm(3,1)=dreal(fact1+fact2)
	    mm(3,2)=dreal(fact1-fact2)/mm(1,1)
	    mm(3,3)=dreal(fact3+fact4)/mm(1,1)
	    mm(3,4)=dimag(c + cc)/mm(1,1)

	    fact1=s4TM(i)*dconjg(s2TM(i))
	    fact3=s1TE(i)*dconjg(s2TM(i))
	    mm(4,1)=dimag(fact1+fact2)
	    mm(4,2)=dimag(fact1-fact2)/mm(1,1)
	    mm(4,3)=dimag(fact3-fact4)/mm(1,1)
	    mm(4,4)=dreal(fact3-fact4)/mm(1,1)
c       write(8,*)int(angle),real(mm(1,1)),real(mm(1,2))
c	print*,'ang, s11=',angledd-inc_ang,real(mm(1,1))
c	write(8,*)real(angledd-inc_ang),real(dlog(mm(1,1)))
       write(8,*)int(angle),real(mm(1,1)),real(mm(1,2)),
     +  real(mm(3,3)),real(mm(3,4))
	endif
c ---------------------------------------------------------------------
c  This endif statement lets us select the angles we want to look at  .
c ---------------------------------------------------------------------
 399	continue 
c	print*,int(angledd),real(mm(1,1))
c       ......................................................
c       .  DONE WITH RUNNING OVER THE OBSERVATION ANGLES     .
c       ......................................................

	print*, ' '
 998   continue
 999    close(4)
       close(8)
	print*,'DONE!'
       stop
       end
