
c	*****************************************************
c	********** THESE ARE THE SUBROUTINES ****************
c	**********   LAST REVISED OCTOBER 25, 1995 **********
c	*****************************************************

      subroutine bessel(n,np,bessj,x)
c .......................................................................
c .subroutine to calculate spherical bessel functions of the first 	.
c .kind from order -1 to order n, the result is stored in the array 	.
c .bessj.  n is related to the size parameter (nbg), and np is the	.
c .physical dimension of the bessj array (isize).  x is a real argument .
c .......................................................................

      implicit double precision (a-h,o-z)
      parameter(iacc=500,bigno=1d100,bigni=1d-100)
      integer n,np
      double precision bessj(-1:np),x,xx,pi
      double precision bjp,bj,factor,ax

      pi=1.0
      pi=4.*datan(pi)
      xx=x-2*pi*int(x/2./pi)
      if (n.lt.3) then 
            n=3
      endif

c .......................................................................
c . If x is less than n, we must use the downward recurrence relation, 	.
c . otherwise our results for bessj will be unstable.  In fact, we will .
c . use the downward alot more than the upward because our x will 	.
c . often be less than n, i.e. the size parameter is less than the order.
c .......................................................................
      if (x.lt.n) then
      ax=dabs(x)
      axx=x-2*pi*int(ax/2./pi)
         if (ax.eq.0) then
            bessj(0)=1.
            do 10 i=1, n
                  bessj(i)=0.
 10         continue
         else
            m=2*((n+int(sqrt(float(iacc*n))))/2)
c ........................................................
c . m is the highest order to calculate; the higher m is .
c . the more accuracy we'll get, but we'll keep n terms	 .
c ........................................................
            do 15 j=0,n
                  bessj(j)=0.
 15         continue
            bjp=0.
            bj=1.
            do 20 j=m,1,-1
                  bjm=(2.*j+1.)/ax*bj-bjp
                  bjp=bj
                  bj=bjm
                  if(dabs(bj).gt.bigno) then
c ..........................................................
c . If bj is huge, we must renormalize to prevent overflow .
c ..........................................................
                        bj=bj*bigni
                        bjp=bjp*bigni
                        do 17 i=0,n
                              bessj(i)=bessj(i)*bigni
 17                     continue
                  endif
                  if (j.le.n) bessj(j)=bjp
 20         continue
            bessj(0)=bj
	    sum=0.
	    do 25 j=0,n
		sum=sum+(2.*j+1.)*bessj(j)*bessj(j)
 25	    continue
            do 30 j=0,n
                  bessj(j)=bessj(j)/dsqrt(sum)
                  if (x.lt.0.and.mod(n,2).eq.1) bessj(j)=-bessj(j) 
 30        continue
            bessj(-1)=dcos(xx)/x
         endif
      
      else
c .......................................................................
c . Use upward recurrence for when x is greater than n			.
c .......................................................................
            bessj(-1)=dcos(xx)/x
            bessj(0)=dsin(xx)/x
            do 40 i=0,n-1
                 bessj(i+1)=bessj(i)*(2.*i+1.)/x-bessj(i-1)
 40         continue
      endif
      return
      end                                         

c	****************************************************

      subroutine c_bessel(n,np,bessj,x)
c .......................................................................
c .subroutine to calculate complex spherical bessel functions of the 	.
c .first kind from order -1 to order n.  The result is stored in array	.
c .bessj.  This is just like the bessel function, except we now have 	.
c .complex arguments, i.e. x is a complex variable	       		.
c .......................................................................

      parameter(iacc=500,bigno=1d100,bigni=1d-100)

	implicit double precision (a-h,o-z)
      double precision pi,xx
      integer n,np
      complex *16 x,xxxx,sum 
      complex *16 bessj(-1:np)
      complex *16 bjp,bj,factor,bjm
      
      if (n.lt.3) then 
            n=3
      endif

      xx=cdabs(x)
      pi=1.0
      pi=4.*datan(pi)
      xxxx=x
      if (xx.gt.32760) then
             xxxx=x-2*pi*int(x/2./pi)
      endif

      if (xx.lt.n) then
c .............................
c . Use downward recurrence   .
c .............................
         if (xx.eq.0) then
            bessj(0)=1.
            do 10 i=1,n
                  bessj(i)=0.
 10         continue
         else
            m=2*((n+int(sqrt(float(iacc*n))))/2)
            do 15 j=0,n
                  bessj(j)=0.
 15         continue
            bjp=dcmplx(0.d0,0.)
            bj=dcmplx(1.d0,1.)
            do 20 j=m,1,-1
                  bjm=(2.*j+1.)/x*bj-bjp
                  bjp=bj
                  bj=bjm
                  xxx=cdabs(bj)
                  if(xxx.gt.bigno) then
c .......................................
c . Renormalize to prevent overflow	.
c .......................................
                        bj=bj*bigni
                        bjp=bjp*bigni
                        do 17 i=0,n
                              bessj(i)=bessj(i)*bigni
 17                     continue
                  endif
                  if (j.le.n) bessj(j)=bjp
 20         continue
            bessj(0)=bj
	    sum=0.
	    do 25 j=0,n
		sum=sum+(2.*j+1.)*bessj(j)*bessj(j)
 25	    continue
            do 30 j=0,n
                  bessj(j)=bessj(j)/cdsqrt(sum)
 30         continue
            bessj(-1)=cdcos(xxxx)/x
        endif

      else
c ...........................
c . Use upward recurrence   .
c ...........................
            bessj(-1)=cdcos(xxxx)/x
            bessj(0)=cdsin(xxxx)/x
            do 40 i=0,n-1
                 bessj(i+1)=bessj(i)*(2.*i+1)/x-bessj(i-1)
 40         continue
      endif
      return
      end                                         

c	****************************************************

      subroutine newman(n,np,bessy,x)
c .......................................................................
c .    subroutine to calculate the spherical bessel function of the	.
c .    second kind (or Newmann function) from order -1			.
c .    to order n; result stored in array bessy.  bess_y  is the common	.
c .    notation for the Newmann function.				.
c .......................................................................

	implicit double precision (a-h,o-z)
      integer n,np
      double precision x,xx,pi
      double precision bessy(-1:np)

      if (n.lt.3) then 
            n=3
      endif
      pi=1.0
      pi=4.*datan(pi)
      xx=x-2.*pi*int(x/2./pi)
c ..................................................................
c .Must use this xx because it's a value between 0 and 2pi, and it .
c .works as the argument for the sin and cos; if we just stuck in  .
c .x instead, it won't work.					   .
c ..................................................................
      bessy(-1)=dsin(xx)/x
      bessy(0)=-dcos(xx)/x
      bessy(1)=-dcos(xx)/x/x-dsin(xx)/x
      do 10 i=2,n
            bessy(i)=(2.*(i-1.)+1.)/x*bessy(i-1)-bessy(i-2)
 10   continue

      return
      end

c	**************************************************

      subroutine c_newman(n,np,bessy,x)
c .......................................................................
c . subroutine to calculate the complex spherical bessel function 	.
c . from order 0 to order n, the result is stored in array bessy.  	.
c . This is the Newman function, aka, bessel function of order 2.	.
c .......................................................................

	implicit double precision(a-h,o-z)

      integer n,np
      complex *16 x,xx,bessy(-1:np)
      double precision pi

      if (n.lt.3) then 
            n=3
      endif
      pi=1.0
      pi=4.*datan(pi)
      xx=x-2.*pi*int(x/2./pi)
      bessy(-1)=cdsin(xx)/x
      bessy(0)=-cdcos(xx)/x
      bessy(1)=-cdcos(xx)/x/x-cdsin(xx)/x
      do 10 i=2,n
            bessy(i)=(2.*(i-1.)+1.)/x*bessy(i-1)-bessy(i-2)
 10   continue
      return
      end

c	**************************************************

      subroutine nplgndr(m,nbr,iisize,legpol,x)
c ...........................................................................
c . Calculate normalized ass. legendre pol. from l=0 to nbr using value of  .
c . -1<=x<=1    Note that everything here is double precision because other .
c . wise we've got a runaway solution.  				    .
c ...........................................................................

      implicit double precision (a-h,o-z)
      double precision x
      integer m,nbr,iisize
      double precision legpol(-2:iisize)
      double precision pmm,somx2,fact,pll,pmmp1     
      mm=mod(m,2)*(-2)+1
      m=abs(m)
      do 5 i=-2,m
            legpol(i)=0.
 5    continue
      pmm = 1.
      if (m.gt.0) then
            somx2=sqrt((1.-x)*(1.+x))
            fact=1.
            do 11 i=1,m
                  pmm=-pmm*fact*somx2/sqrt(real(i))
                  fact=fact+2.
 11         continue
            do 110 i=m+1,2*m
                  pmm=pmm/sqrt(real(i))
 110        continue
            pmm=pmm*sqrt(2.*m+1.)
      endif
      legpol(m)=pmm*mm
      if (nbr.gt.m) then
            pmmp1=x*sqrt(2.*m+3.)*pmm
            legpol(m+1)=pmmp1*mm
      endif
      if (nbr.gt.m+1) then
            do 12 ll=m+2,nbr
                  pll=x*sqrt(2.*ll-1.)*pmmp1
                  pll=pll-sqrt((ll+m-1.)*(ll-m-1.)/(2.*ll-3.))*pmm
                  pll=pll/sqrt((ll-m)*(ll+m)/(2.*ll+1.))
                  pmm=pmmp1
                  pmmp1=pll
                  legpol(ll)=pll*mm
 12         continue
      endif
      m=m*mm
      return
      end                  


c	**************************************************

	subroutine riccati(nbg,hval,zeta,dzeta,x)
	integer nbg,i
	complex *16 hval(-1:nbg),zeta(-1:nbg),dzeta(-1:nbg),x

	zeta(-1)=x*hval(-1)
	do 10 i=0,nbg
		zeta(i)=x*hval(i)
		dzeta(i)=x*hval(i-1) -i*hval(i)
 10	continue
	return
	end

c	**************************************************

      subroutine ludcmp(a,n,np,indx,d)
c .......................................................................
c . given nxn matrix a with physical dim NP, this routine replaces it	.
c . by the LU decomposition of a rowwise permutation of itself.  A and  .
c . n are input. A is output. INDX is an output vector which records the.
c . row permutation effected by the partial pivoting; D is output	.
c . as +-1 depending on whether the number of row interchanges		.
c . was even or odd.  This routine is used in combination with Lubksb	.
c . to solve linear equations or invert a matrix.			.
c .......................................................................

      complex *16 tiny
      parameter (nmax=350)
c ....................................
c . this nmax value should be at     .
c . least twice the value of nbg     .
c ....................................
      complex *16 a(np,np),sum,cdum,vv(nmax),aamax
      dimension indx(n)


      tiny=dcmplx(1.0d-80,1.0d-80)
      d=1.
      do 12 i=1,n
            aamax=dcmplx(0.d0,0.)
            do 11 j=1,n
                  if (cdabs(a(i,j)).gt.cdabs(aamax)) aamax=a(i,j)
 11         continue
            if (cdabs(aamax).eq.0) pause 'singular matrix'
            vv(i)=1./aamax
 12   continue
      do 19 j=1,n
            do 14 i=1,j-1
                  sum=a(i,j)
                  do 13 k=1,i-1
                        sum=sum-a(i,k)*a(k,j)
 13               continue
                  a(i,j)=sum
 14         continue
            aamax=dcmplx(0.d0,0.)
            do 16 i=j,n
                  sum=a(i,j)
                  do 15 k=1,j-1
                        sum=sum-a(i,k)*a(k,j)
 15               continue
                  a(i,j)=sum
                  cdum=vv(i)*sum
                  if (cdabs(cdum).ge.cdabs(aamax)) then
                        imax=i    
                        aamax=cdum
                  endif
 16         continue
            if (j.ne.imax) then
                  do 17 k=1,n
                        cdum=a(imax,k)
                        a(imax,k)=a(j,k)
                        a(j,k)=cdum
 17               continue
                  d=-d
                  vv(imax)=vv(j)
            endif
            indx(j)=imax
            if(cdabs(a(j,j)).eq.0) a(j,j)=tiny
            if (j.ne.n) then
                  cdum=1./a(j,j)
                  do 18 i=j+1,n
                        a(i,j)=a(i,j)*cdum
 18               continue
            endif
 19   continue
      return
      end

c	**************************************************

      subroutine lubksb(a,n,np,indx,b)
c .......................................................................
c . Solves the set of n linear equations A.X=B. Here A is input, not	.
c . as the matrix A but rather as its LU decomp, from LUDCMP.  INDX	.
c . is input as the permutation vector returned by LUDCMP. B is input	.
c . as the right-hand side vector B, and returns with the solution 	.
c . Vector X.  A,N,NP and INDX are not modified by this routine and 	.
c . can be left in place for successive calls with different right-hand	.
c . sides B.  This routine takes into account the possibility that B 	.
c . can begin with many zero elements, so it is efficient for use in 	.
c . matrix inversion.							.
c .......................................................................
      
      complex *16 a(np,np),b(n),sum
      dimension indx(n)

      ii=0
      
      do 12 i=1,n
            ll=indx(i)
            sum=b(ll)
            b(ll)=b(i)
            if (ii.ne.0) then
                  do 11 j=ii,i-1
                        sum=sum-a(i,j)*b(j)
 11               continue
            else if (cdabs(sum).ne.0.) then
                  ii=i
            endif
            b(i)=sum
 12   continue
      do 14 i=n,1,-1
            sum=b(i)
            if (i.lt.n) then
                  do 13 j=i+1,n
                        sum=sum-a(i,j)*b(j)
 13               continue
            endif
            b(i)=sum/a(i,i)
 14   continue
      return
      end

c	**************************************************

      subroutine mprove(a,alud,n,np,indx,b,x)
c ......................................................................
c . improves a solution vector X of the linear set of equations A*X=B  .
c . The matrix A, and the vectors B and X are input, as is the dim     .	
c . n.  Also input is alud the ludcomp of A as returned by ludcmp, and .
c . the vector indx also returned by that routine.  On output, only    .
c . X is modified, to an improved set of values.		       .
c ......................................................................

      parameter (nmax=350)
c ....................................................
c . all the values here should be the same as those  .
c . found in the ludcmp subroutine, eg nmax should   .
c . have the same numerical value as in ludcmp	     .
c ....................................................
      complex *16 a(np,np),alud(np,np),b(n),x(n),sdp,r(nmax)
      dimension indx(n)        


      do 12 i=1,n
            sdp=-b(i)
            do 11 j=1,n
                  sdp=sdp+a(i,j)*x(j)
 11         continue
            r(i)=sdp
 12   continue
      call lubksb(alud,n,np,indx,r)
      do 13 i=1,n
            x(i)=x(i)-r(i)
 13   continue
      return
      end  


